//
//  JTAPIConnection.java
//  JTAPIConnection
//
//  Created by Abdelatif Nouqrat on Mon Mar 15 2004.
//  Copyright (c) 2004 ilink GmbH. All rights reserved.
//
package de.ilink.jtapi.makecall;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import javax.telephony.Address;
import javax.telephony.Call;
import javax.telephony.JtapiPeerFactory;
import javax.telephony.Provider;
import javax.telephony.ProviderObserver;
import javax.telephony.Terminal;
import javax.telephony.callcontrol.CallControlCall;
import javax.telephony.callcontrol.CallControlCallObserver;
import javax.telephony.callcontrol.events.CallCtlConnEstablishedEv;
import javax.telephony.callcontrol.events.CallCtlConnOfferedEv;
import javax.telephony.callcontrol.events.CallCtlTermConnTalkingEv;
import javax.telephony.events.CallEv;
import javax.telephony.events.ProvEv;
import javax.telephony.events.ProvInServiceEv;
import javax.telephony.events.TermConnDroppedEv;

/**
 * Connection between the application and the JTAPI server (TeamCall Server). Provides 
 * functionality for monitoring a device and makes a call with this device.
 * <br><br>
 * Fires property change events if the call or the provider changes.
 *
 * @author Abdelatif Nouqrat
 */
public class JTAPIConnection 
implements CallControlCallObserver, ProviderObserver
{
	private String fDevice;
	private String fProviderString;
	private Provider fProvider;
	private Address fOrigAddress;
	private Terminal fOrigTerminal;
	private Call fCall = null;
	
	private PropertyChangeSupport fSupport = new PropertyChangeSupport(this);
	
	/**
	 * Constructs a JTAPIConnection with the given provider string. 
	 *
	 * TeamCall JTAPI needs a provider string like "&lt;port&gt;@&lt;IP or hostname of teamcall&gt;;login=&lt;username&gt;;passwd=&lt;password&gt;"
	 */
	public JTAPIConnection (String aProviderString)
	{
		fProviderString = aProviderString;	
	}
	
	/**
	 * Connect JTAPIConnection to the TeamCall Server. You have to call this function before
	 * you can monitor a device.
	 */
	public void initializeProvider()
	{
		try
		{
			fProvider = JtapiPeerFactory.getJtapiPeer(Constants.TEAMCALL_JTAPI_PEER_CLASS).getProvider(fProviderString);
			fProvider.addObserver(this);
		} catch (Exception e)
		{
			System.out.println("Can't get Provider: " + e.toString());
		}
	}
	
	/**
	 * Set a monitor for the given device (a dialing number). 
	 * The user must be assoziated with this device.
	 */
	public void monitorStartForDevice(String aDevice)
	{
		fDevice = aDevice;
		try
		{
			fOrigAddress = fProvider.getAddress(fDevice);
			fOrigAddress.addCallObserver(this);
		
			/* Just get some Terminal on this Address */
			Terminal[] terminals = fOrigAddress.getTerminals();
			
			if (terminals == null)
			{
				System.out.println("No Terminals on Address.");
			}
			fOrigTerminal = terminals[0];
		}
		catch (Exception excp)
		{
			excp.printStackTrace();
		}
	}
	
	/**
	 * Place a call to the given destination.
	 * @param aDestination The destination daling number.
	 */
	public void makeCall (String aDestination)
	{
		
		/*
		 * Create the telephone call object.
		 */
		try
		{
			fCall = fProvider.createCall();
		} catch (Exception excp)
		{
			System.out.println("Could not create call");
			return;
		}

		/*
		 * Place the telephone call.
		 */
		try
		{
			fCall.connect(fOrigTerminal, fOrigAddress, aDestination);
		}
		catch (Exception excp)
		{
			System.out.println("Could not connect to " + aDestination);
			try
			{
				((CallControlCall)fCall).drop();
				fCall = null;
			}
			catch (Exception ex)
			{
			}
		}
	}
    
	/**
	 * Clears the currently active connection. Does nothing if there is no connection.
	 */
	public void drop ()
	{
		if (null == fCall)
			return;
		
		try
		{
			((CallControlCall)fCall).drop();
		}
		catch (Exception ex)
		{
			System.out.println("Could not drop");
		}
		fCall = null;
	}
	
	
	/**
	 * Call back function for the CallControlCallObserver
	 */
	public void callChangedEvent(CallEv[] arg0)
	{
		for (int i = 0; i < arg0.length; i++)
		{
			// System.out.println("Call Changed Event: " + arg0[i].getClass().getName()) ;
			
			if (arg0[i] instanceof CallCtlTermConnTalkingEv)
			{
				firePropertyChange(new PropertyChangeEvent(this, Constants.PROPERTY_NAME_TALKING, new Integer(0), new Integer(1)));
			}
			
			else if (arg0[i] instanceof TermConnDroppedEv)
			{
				firePropertyChange(new PropertyChangeEvent(this, Constants.PROPERTY_NAME_DROPPED, new Integer(0), new Integer(1)));
			}
			else if (arg0[i] instanceof CallCtlConnEstablishedEv)
			{
				firePropertyChange(new PropertyChangeEvent(this, Constants.PROPERTY_NAME_ESTABLISHED, new Integer(0), new Integer(1)));
			}
			
			else if (arg0[i] instanceof CallCtlConnOfferedEv)
			{
				firePropertyChange(new PropertyChangeEvent(this, Constants.PROPERTY_NAME_OFFERED, new Integer(0), new Integer(1)));
			}
		}
		
	}
	
	/**
	 * Call back function for the ProviderObserver interface
	 */
	public void providerChangedEvent(ProvEv[] arg0)
	{
		for (int i = 0; i < arg0.length; i++)
		{
			if (arg0[i] instanceof ProvInServiceEv)
			{
				firePropertyChange (new PropertyChangeEvent(this, Constants.PROPERTY_NAME_PROVIDER_IN_SERVICE, new Integer(0), new Integer(1)));
			}
		}
		
	}
	
	/**
	 * Fired every time the Provider or the Call changed.  
	 */
	public void firePropertyChange(PropertyChangeEvent e)
	{
        fSupport.firePropertyChange(e);
    }
	
    public void addPropertyChangeListener(PropertyChangeListener l)
	{
		fSupport.addPropertyChangeListener(l);
    }
	
    public void removePropertyChangeListener(PropertyChangeListener l)
	{
        fSupport.removePropertyChangeListener(l);
    }
}

