package de.ilink.examples;

import java.util.*;
import java.text.*;
import javax.telephony.*;

import java.awt.*;
import javax.swing.*;

/**
 * The CallListener class implements the TerminalConectionListener 
 * and receives all Call-related events.
 * @author nouqrat
 */

public class CallListener
implements TerminalConnectionListener
{
	private static final DateFormat DATE_FORMATTER =
		new SimpleDateFormat("yyyy-MM-dd HH:mm:ss") ;
		
	private Terminal fTerminal = null;
	private String fAddressName = null;
	private JButton fButton = null;
	private Color fStatus = Color.green;
	// idle(green) / talking(red) / incomingalerting(yellow)

	private boolean fTerminalConnectionEventEnabled = false;
	private boolean fConnectionEventEnabled = false;
	private boolean fCallEventEnabled = false;
	private boolean fMetaEventEnabled = false;

	/**
	 * constructor
	 * @param aProvider a Provider 
	 * @param anAddressName Address to be monitored
	 */
	CallListener(Provider aProvider, String anAddressName)
	{
		init(aProvider, anAddressName, null);
	}

	/**
	 * constructor
	 * @param aProvider a provider
	 * @param anAddressName Address to be monitored
	 * @param aButton button that displays the status of the local connection
	 * 		  on the monitored address 
	 */

	CallListener(
		Provider aProvider,
		String anAddressName,
		JButton aButton)
	{
		init(aProvider, anAddressName, aButton);

	}

	

	/**
	 * init() Initializes the fields fAddressName, fButton, fTerminal  
	 * and the 4 fields fTerminalConnectionEventEnabled, fConnectionEventEnabled, 
	 * fCallEventEnabled, fMetaEventEnabled
	 * @param aProvider a provider
	 * @param anAddressName Address to be monitored 
	 * @param aButton button that displays the status of the local connection
	 * 				  on the monitored address
	 */
	public void init(
		Provider aProvider,
		String anAddressName,
		JButton aButton)
	{
		fAddressName = anAddressName;
		fButton = aButton;

		try
		{
			fTerminal = aProvider.getTerminal(fAddressName);
			// javax.telephony.InvalidArgumentException
			fTerminal.addCallListener(this);
			// may throw javax.telephony.ResourceUnavailableException
		} catch (Exception anExc)
		{
			
			System.out.println(anExc.getMessage().toString());
		}

		fTerminalConnectionEventEnabled = false;
		fConnectionEventEnabled = false;
		fCallEventEnabled = false;
		fMetaEventEnabled = false;

	}
	/**
	 * connectionStateToString() converts a connection state to a string 
	 * @param aState state of a local connection
	 * @return connStateString state of a connection as a string
	 */
	static public String connectionStateToString(int aState)
	{
		int[] connectionStatesAsInt =
			{
				Connection.IDLE,
				Connection.DISCONNECTED,
				Connection.INPROGRESS,
				Connection.ALERTING,
				Connection.CONNECTED,
				Connection.UNKNOWN,
				Connection.FAILED };
		String[] connectionStatesAsString =
			{
				"IDLE",
				"DISCONNECTED",
				"INPROGRESS",
				"ALERTING",
				"CONNECTED",
				"UNKNOWN",
				"FAILED" };

		String connStateString = "--fatal--";
		for (int i = 0; i < connectionStatesAsInt.length; i++)
		{
			if (aState == connectionStatesAsInt[i])
			{
				connStateString = connectionStatesAsString[i];
			}
		}
		return connStateString + "(" + aState + ")";
	}
	/**
	 * callStateToString() converts a callState to a String
	 * @param aState state of a call
	 * @return callStateString representation of a call as a string
	 */
	static public String callStateToString(int aState)
	{
		int[] callStatesAsInt = { Call.IDLE, Call.ACTIVE, Call.INVALID };
		String[] callStatesAsString = { "IDLE", "ACTIVE", "INVALID" };

		String callStateString = "--undefined--";
		for (int i = 0; i < callStatesAsInt.length; i++)
		{
			if (aState == callStatesAsInt[i])
			{
				callStateString = callStatesAsString[i];
			}
		}
		return callStateString + "(" + aState + ")";
	}

	/**
	 * logCallInfo() logs the call information 
	 * @param aLocalConnection a local connection
	 */
	public void logCallInfo(Connection aLocalConnection)
	{
		Call call = aLocalConnection.getCall();
		Connection[] connArr = call.getConnections();
		System.out.print(
			"call=<"
				+ call
				+ "> callstate="
				+ callStateToString(call.getState())
				+ " "
				+ connArr.length);
		System.out.print(" [");
		int nRemoteConn = 0;
		for (int connEnum = 0; connEnum < connArr.length; connEnum++)
		{
			Connection conn = connArr[connEnum];
			if (conn != aLocalConnection)
			{
				if (nRemoteConn != 0)
				{
					System.out.print(",");
				}
				nRemoteConn++;
				Address addr = conn.getAddress();
				String addressName = addr.getName();
				System.out.print(
					connectionStateToString(conn.getState())
						+ "/"
						+ addressName);
			}
		}
		System.out.print("]");
	}
	/**
	 * logTerminalInfo() logs the terminal information   
	 * @param eventType type of incoming event
	 */
	public void logTerminalInfo(String eventType)
	{
		
		Date date = new Date();
		String dateString = DATE_FORMATTER.format(date);
		// log information about terminal:
		System.out.println(
			dateString
				+ ": "
				+ fTerminal
				+ " "
				+ eventType
				+ " "
				+ fTerminal.getName());
		// log information about addresses of terminal:
		Address[] addrArr = fTerminal.getAddresses();
		for (int addrEnum = 0; addrEnum < addrArr.length; addrEnum++)
		{
			Address addr = addrArr[addrEnum];
			// log information about local connections and call of address:
			Connection[] connArr = addr.getConnections();
			System.out.println("  " + addr.getName() + " " + connArr.length);
			for (int connEnum = 0; connEnum < connArr.length; connEnum++)
			{
				Connection localconn = connArr[connEnum];
				System.out.print(
					"    localstate="
						+ connectionStateToString(localconn.getState())
						+ " ");
				logCallInfo(localconn);
				System.out.println();
			}
		}
	}

	/**
	 * updateStatus() updates the status of the connection
	 * and signals the new status through color modification 
	 * red 		not Disconnected status  
	 * yellow	alerting status
	 * green	Disconnected status
	 */
	public void updateStatus()
	{
		boolean alerting = false;
		boolean talking = false;
		boolean pickUpButtonStatus = false;
		Color newStatus = fStatus;
		Address[] addrArr = fTerminal.getAddresses();
		for (int addrEnum = 0; addrEnum < addrArr.length; addrEnum++)
		{
			Address addr = addrArr[addrEnum];
			Connection[] connArr = addr.getConnections();
			if (connArr != null)
			{
				for (int connEnum = 0; connEnum < connArr.length; connEnum++)
				{
					Connection localconn = connArr[connEnum];
					int localstate = localconn.getState();
					if (localstate == Connection.ALERTING)
					{
						alerting = true;
					} else if (localstate != Connection.DISCONNECTED)
					{
						talking = true;

					}
				}

				if (alerting && !talking)
				{
					newStatus = Color.yellow;
					

				} else if (talking)
				{
					newStatus = Color.red;
					
				} else
				{
					newStatus = Color.green;
					
				}
			} else
			{
				newStatus = Color.green;
				
			}
			if (newStatus != fStatus)
			{
				fStatus = newStatus;
				if (fButton != null)
				{
					fButton.setBackground(newStatus);
					
				} else
				{
					System.out.println(addr.getName() + " Status = " + fStatus);
				}

			}
		}
	}

	/**
	 * terminalConnectionEvent() logs terminal information when a terminalConnectionEvent is enabled 
	 * @param event a terminalConnectionEvent
	 * @param eventType eventType as a String for logging terminal information
	 */
	public void terminalConnectionEvent(
		TerminalConnectionEvent event,
		String eventType)
	{
		if (fTerminalConnectionEventEnabled)
		{
			logTerminalInfo(eventType);
		}
	}

	/**
	 * connectionEvent() logs terminal information when a ConnectionEvent is enabled 
	 * @param event a ConnectionEvent 
	 * @param eventType eventType as a String for logging terminal information
	 */
	public void connectionEvent(ConnectionEvent event, String eventType)
	{
		if (fConnectionEventEnabled)
		{
			logTerminalInfo(eventType);
		}
		updateStatus();

	}

	/**
	 * callEvent() logs terminal information when a CallEvent is enabled
	 * @param event a CallEvent
	 * @param eventType eventType as a String for logging terminal information
	 */
	public void callEvent(CallEvent event, String eventType)
	{
		if (fCallEventEnabled)
		{
			logTerminalInfo(eventType);

		}
	}

	/**
	 * metaEvent() logs terminal information when a MetaEvent is enabled
	 * @param event a MetaEvent
	 * @param eventType eventType as a String for logging terminal information
	 */
	public void metaEvent(MetaEvent event, String eventType)
	{

		if (fMetaEventEnabled)
		{
			logTerminalInfo(eventType);
		}
		
	}

	/**
	 * 
	 * @see javax.telephony.TerminalConnectionListener
	 */
	public void terminalConnectionActive(TerminalConnectionEvent event)
	{
		terminalConnectionEvent(event, "terminalConnectionActive");
	}
	public void terminalConnectionCreated(TerminalConnectionEvent event)
	{
		terminalConnectionEvent(event, "terminalConnectionCreated");
	}
	public void terminalConnectionDropped(TerminalConnectionEvent event)
	{
		terminalConnectionEvent(event, "terminalConnectionDropped");
	}
	public void terminalConnectionPassive(TerminalConnectionEvent event)
	{
		terminalConnectionEvent(event, "terminalConnectionPassive");
	}
	public void terminalConnectionRinging(TerminalConnectionEvent event)
	{
		terminalConnectionEvent(event, "terminalConnectionRinging");
	}
	public void terminalConnectionUnknown(TerminalConnectionEvent event)
	{
		terminalConnectionEvent(event, "terminalConnectionUnknown");
	}
	/**
	 * 
	 * @see javax.telephony.ConnectionListener
	 */
	public void connectionAlerting(ConnectionEvent event)
	{
		connectionEvent(event, "connectionAlerting");
	}
	public void connectionConnected(ConnectionEvent event)
	{
		connectionEvent(event, "connectionConnected");
	}
	public void connectionCreated(ConnectionEvent event)
	{
		connectionEvent(event, "connectionCreated");
	}
	public void connectionDisconnected(ConnectionEvent event)
	{
		connectionEvent(event, "connectionDisconnected");
	}
	public void connectionFailed(ConnectionEvent event)
	{
		connectionEvent(event, "connectionFailed");
	}
	public void connectionInProgress(ConnectionEvent event)
	{
		connectionEvent(event, "connectionInProgress");
	}
	public void connectionUnknown(ConnectionEvent event)
	{
		connectionEvent(event, "connectionUnknown");
	}

	/**
	 *  
	 * @see javax.telephony.CallListener
	 */
	public void callActive(CallEvent event)
	{
		callEvent(event, "callActive");
	}
	public void callInvalid(CallEvent event)
	{
		callEvent(event, "callInvalid");
	}
	public void callEventTransmissionEnded(CallEvent event)
	{
		callEvent(event, "callEventTransmissionEnded");
	}

	public void singleCallMetaProgressStarted(MetaEvent event)
	{
		metaEvent(event, "singleCallMetaProgressStarted");
	}
	public void singleCallMetaProgressEnded(MetaEvent event)
	{
		metaEvent(event, "singleCallMetaProgressEnded");
	}
	public void singleCallMetaSnapshotStarted(MetaEvent event)
	{
		metaEvent(event, "singleCallMetaSnapshotStarted");
	}
	public void singleCallMetaSnapshotEnded(MetaEvent event)
	{
		metaEvent(event, "singleCallMetaSnapshotEnded");
	}

	public void multiCallMetaMergeStarted(MetaEvent event)
	{
		metaEvent(event, "multiCallMetaMergeStarted");
	}
	public void multiCallMetaMergeEnded(MetaEvent event)
	{
		metaEvent(event, "multiCallMetaMergeEnded");
	}
	public void multiCallMetaTransferStarted(MetaEvent event)
	{
		metaEvent(event, "multiCallMetaTransferStarted");
	}
	public void multiCallMetaTransferEnded(MetaEvent event)
	{
		metaEvent(event, "multiCallMetaTransferEnded");
	}

}
